@@ -1,19 +1,19 @@
 /*
-**	pjl library
+**	PJL C++ Library
 **	fake_ansi.h -- Fake some ANSI C++ comittee resolutions
 **
-**	Copyright (C) 1998  Paul J. Lucas
+**	Copyright (C) 2001  Paul J. Lucas
 **
 **	This program is free software; you can redistribute it and/or modify
 **	it under the terms of the GNU General Public License as published by
 **	the Free Software Foundation; either version 2 of the Licence, or
 **	(at your option) any later version.
-** 
+**
 **	This program is distributed in the hope that it will be useful,
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **	GNU General Public License for more details.
-** 
+**
 **	You should have received a copy of the GNU General Public License
 **	along with this program; if not, write to the Free Software
 **	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -22,23 +22,13 @@
 #ifndef	fake_ansi_H
 #define	fake_ansi_H
 
+// standard
+#include <cstddef>			/* for ptrdiff_t */
+
+// local
 #include "platform.h"
 
-/* new cast operators */
-
-#ifdef	PJL_NO_NEW_CASTS
-#	define CONST_CAST(T)		(T)
-#	define STATIC_CAST(T)		(T)
-#	define DYNAMIC_CAST(T)		(T)
-#	define REINTERPRET_CAST(T)	(T)
-#else
-#	define CONST_CAST(T)		const_cast<T>
-#	define STATIC_CAST(T)		static_cast<T>
-#	define DYNAMIC_CAST(T)		dynamic_cast<T>
-#	define REINTERPRET_CAST(T)	reinterpret_cast<T>
-#endif
-
-/* namespaces */
+////////// namespaces /////////////////////////////////////////////////////////
 
 #ifdef	PJL_NO_NAMESPACES
 #	define namespace		/* nothing */
@@ -46,14 +36,32 @@
 #	define std			/* nothing */
 #endif
 
-/* some new keywords */
+//
+// The HP STL headers are broken for g++ 2.95.x and earlier because
+// stl_config.h incorrectly defines __STL_NO_NAMESPACES when it shouldn't
+// (causing __STL_USE_NAMESPACES to be undefined) and stl_iterator.h defines
+// "struct iterator" only when __STL_USE_NAMESPACES is defined.  This is fixed
+// in 2.96, but for earlier versions we define std::iterator here ourselves:
+// the definition is copied/pasted from stl_iterator.h.
+//
+#ifdef	PJL_GCC_295
+namespace std {
+	template <class _Category, class _Tp, class _Distance = ptrdiff_t,
+		  class _Pointer = _Tp*, class _Reference = _Tp&>
+	struct iterator {
+		typedef _Category  iterator_category;
+		typedef _Tp        value_type;
+		typedef _Distance  difference_type;
+		typedef _Pointer   pointer;
+		typedef _Reference reference;
+	};
+}
+#endif
+
+////////// some new keywords //////////////////////////////////////////////////
 
 #ifdef	PJL_NO_EXPLICIT
 #	define explicit			/* nothing */
 #endif
 
-#ifdef	PJL_NO_MUTABLE
-#	define mutable			/* nothing */
-#endif
-
 #endif	/* fake_ansi_H */
