@@ -8,12 +8,12 @@
 **	it under the terms of the GNU General Public License as published by
 **	the Free Software Foundation; either version 2 of the License, or
 **	(at your option) any later version.
-** 
+**
 **	This program is distributed in the hope that it will be useful,
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **	GNU General Public License for more details.
-** 
+**
 **	You should have received a copy of the GNU General Public License
 **	along with this program; if not, write to the Free Software
 **	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -23,7 +23,7 @@
 #include <cctype>
 
 // local
-#include "fake_ansi.h"				/* for CONST_CAST */
+#include "bcd.h"
 #include "file_list.h"
 
 //*****************************************************************************
@@ -43,16 +43,27 @@
 //
 //*****************************************************************************
 {
-#ifdef	PJL_NO_MUTABLE
-#	define THIS	( CONST_CAST( file_list* )( this ) )
-#else
-#	define THIS	this
-#endif
-	THIS->size_ = 0;
-	for ( register char const *c = ptr; *c; ++c )
-		if ( *c == ' ' )
-			++THIS->size_;
-	return THIS->size_ /= 2;
+	size_ = 0;
+	//
+	// It would be nice if there were a way to calculate the size of the
+	// file list other than by just marching though it.  Since this should
+	// be as fast as possible, a much simplified version of the parse_bcd()
+	// code has been inlined here by hand -- twice.  (We also don't care
+	// what the actual numbers are, so there's no point in computing them,
+	// so we save having to do two multiplies, adds, shifts, and logical
+	// ands for each file.)
+	//
+	for ( register unsigned char const *p = ptr_; *p != 0xFF; ) {
+		while ( (*p++ & 0x0F) != 0x0A ) ;	// skip past file index
+		if ( *p == 0xEE ) {			// META ID list follows
+			while ( *++p != 0xEE ) ;
+			++p;				// skip past 0xEE
+		}
+		while ( (*p++ & 0x0F) != 0x0A ) ;	// skip past occurrences
+		while ( (*p++ & 0x0F) != 0x0A ) ;	// skip past rank
+		++size_;
+	}
+	return size_;
 }
 
 //*****************************************************************************
@@ -71,36 +82,32 @@
 //
 // SEE ALSO
 //
-//	word_index.h
 //	index.c		write_full_index() for a description of the index file
 //			format.
 //
 //*****************************************************************************
 {
-	if ( !*c_ ) {
+	if ( *c_ == 0xFF ) {
 		//
-		// Reached end of null-terminated list: set iterator to end.
+		// Reached the end of the list: set iterator to end.
 		//
 		c_ = 0;
 		return *this;
 	}
 
-	v_.index_ = 0;
-	while ( isdigit( *c_ ) ) v_.index_ = v_.index_ * 10 + *c_++ - '0';
+	v_.index_ = parse_bcd( c_ );
 
 	if ( !v_.meta_ids_.empty() )
 		v_.meta_ids_.erase( v_.meta_ids_.begin(), v_.meta_ids_.end() );
-	while ( *c_ == '\2' ) {			// is a META ID
-		++c_;
-		register int meta_id = 0;
-		while ( isdigit( *c_ ) ) meta_id = meta_id * 10 + *c_++ - '0';
-		v_.meta_ids_.insert( meta_id );
+	if ( *c_ == 0xEE ) {			// META ID list follows
+		++c_;				// skip past 0xEE
+		while ( *c_ != 0xEE )
+			v_.meta_ids_.insert( parse_bcd( c_ ) );
+		++c_;				// skip past 0xEE
 	}
-	++c_;					// skip past non-digit
 
-	v_.rank_ = 0;
-	while ( *c_ != '\1' ) v_.rank_  = v_.rank_  * 10 + *c_++ - '0';
-	++c_;					// skip past non-digit
+	v_.occurrences_ = parse_bcd( c_ );
+	v_.rank_ = parse_bcd( c_ );
 
 	return *this;
 }
