@@ -8,12 +8,12 @@
 **	it under the terms of the GNU General Public License as published by
 **	the Free Software Foundation; either version 2 of the License, or
 **	(at your option) any later version.
-** 
+**
 **	This program is distributed in the hope that it will be useful,
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **	GNU General Public License for more details.
-** 
+**
 **	You should have received a copy of the GNU General Public License
 **	along with this program; if not, write to the Free Software
 **	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -23,13 +23,12 @@
 #define	file_list_H
 
 // standard
-#include <cstddef>				/* for ptrdiff_t */
+#include <cstddef>			/* for ptrdiff_t */
 #include <iterator>
-#include <sys/types.h>				/* for off_t */
 
 // local
-#include "fake_ansi.h"				/* for mutable */
-#include "word_index.h"
+#include "fake_ansi.h"			/* for iterator, std */
+#include "index_segment.h"
 #include "word_info.h"
 
 //*****************************************************************************
@@ -40,13 +39,13 @@
 //
 // DESCRIPTION
 //
-//	This class, given a word_index::const_iterator, accesses the list of
+//	This class, given a index_segment::const_iterator, accesses the list of
 //	files the word is in.  Once an instance is created, the list of files
 //	can be iterated over.
 //
 // SEE ALSO
 //
-//	word_index.h
+//	index_segment.h
 //	word_info.h
 //	index.c		write_full_index() for a description of the index file
 //			format.
@@ -60,17 +59,16 @@ public:
 	typedef ptrdiff_t difference_type;
 
 	typedef word_info::file value_type;
-	typedef value_type* pointer;
 	typedef value_type const* const_pointer;
-	typedef value_type& reference;
 	typedef value_type const& const_reference;
 
 	////////// constructors ///////////////////////////////////////////////
 
-	file_list( word_index::const_iterator const &iter ) :
-		ptr( *iter ), size_( -1 )
+	file_list( index_segment::const_iterator const &iter ) :
+		ptr_( reinterpret_cast<unsigned char const*>( *iter ) ),
+		size_( -1 )			// -1 = "haven't computed yet"
 	{
-		while ( *ptr++ ) ;		// skip past word
+		while ( *ptr_++ ) ;		// skip past word
 	}
 
 	////////// member functions ///////////////////////////////////////////
@@ -83,7 +81,7 @@ public:
 	friend class const_iterator;
 
 	class const_iterator :
-		public forward_iterator< value_type, difference_type > {
+		public std::iterator< std::forward_iterator_tag, value_type > {
 	public:
 		const_iterator() { }
 
@@ -107,19 +105,19 @@ public:
 		}
 
 	private:
-		const_iterator( char const *p ) : c_( p ) {
+		const_iterator( unsigned char const *p ) : c_( p ) {
 			if ( c_ ) operator++();
 		}
-		char const *c_;
+		unsigned char const *c_;
 		value_type v_;
 		friend class file_list;
 	};
 
-	const_iterator begin() const	{ return const_iterator( ptr ); }
+	const_iterator begin() const	{ return const_iterator( ptr_ ); }
 	const_iterator end() const	{ return const_iterator( 0 ); }
 private:
-	char const *ptr;
-	mutable size_type size_;
+	unsigned char const	*ptr_;
+	mutable size_type	size_;
 
 	size_type calc_size() const;
 };
