@@ -8,12 +8,12 @@
 **	it under the terms of the GNU General Public License as published by
 **	the Free Software Foundation; either version 2 of the License, or
 **	(at your option) any later version.
-** 
+**
 **	This program is distributed in the hope that it will be useful,
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **	GNU General Public License for more details.
-** 
+**
 **	You should have received a copy of the GNU General Public License
 **	along with this program; if not, write to the Free Software
 **	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -25,11 +25,15 @@
 // local
 #include "config.h"
 #include "exit_codes.h"
-#include "file_vector.h"
+#include "index_segment.h"
+#include "mmap_file.h"
+#include "platform.h"
 #include "stop_words.h"
 #include "util.h"
+#include "word_util.h"
 
 #ifndef	PJL_NO_NAMESPACES
+using namespace PJL;
 using namespace std;
 #endif
 
@@ -45,7 +49,14 @@ stop_word_set*		stop_words;		// pointer to global set
 //
 // DESCRIPTION
 //
-//	Construct (initialize) a stop_word_set.
+//	Construct (initialize) a stop_word_set either from the built-in list of
+//	stop-words or from a list given in a file.
+//
+// PARAMETERS
+//
+//	file_name	The name of the file containing a list of stop-words.
+//			If it's either a null pointer or the string it points
+//			to is a null string, then the built-in list is used.
 //
 //*****************************************************************************
 {
@@ -209,7 +220,7 @@ stop_word_set*		stop_words;		// pointer to global set
 		"most", "mostly",
 		"mrs",
 		"much",
-		"must",
+		"must", "mustn't",
 		"my",
 		"myself",
 
@@ -265,6 +276,7 @@ stop_word_set*		stop_words;		// pointer to global set
 		"seem", "seemed", "seeming", "seems",
 		"seven", "seventeen", "seventy",
 		"several",
+		"shall",
 		"she",
 		"should", "shouldn", "shouldn't",
 		"since",
@@ -337,8 +349,7 @@ stop_word_set*		stop_words;		// pointer to global set
 		"what", "whatever",
 		"when", "whence", "whenever",
 		"where", "whereafter", "whereas", "whereby",
-		"wherein", "whereupon",
-		"wherever",
+		"wherein", "whereupon", "wherever",
 		"whether",
 		"which",
 		"while",
@@ -359,7 +370,8 @@ stop_word_set*		stop_words;		// pointer to global set
 
 		"yes",
 		"yet",
-		"you", "your", "yours", "yourself", "yourselves",
+		"you", "you'll", "your", "you're", "yours",
+		"yourself", "yourselves",
 
 		// Z
 
@@ -376,38 +388,40 @@ stop_word_set*		stop_words;		// pointer to global set
 		return;
 	}
 
-	file_vector<char> file( file_name );
+	mmap_file const file( file_name );
 	if ( !file ) {
-		ERROR << "could not open \"" << file_name << '"' << endl;
+		error() << "could not open \"" << file_name << "\"\n";
 		::exit( Exit_No_Read_Stopwords );
 	}
 
-	char word_buf[ Word_Hard_Max_Size + 1 ];
-	int word_len;
-	bool in_word = false;
+	char	word[ Word_Hard_Max_Size + 1 ];
+	int	len;
+	bool	in_word = false;
 
-	register file_vector<char>::const_iterator c = file.begin();
-	while ( 1 ) {
+	register mmap_file::const_iterator c = file.begin();
+	while ( true ) {
 		if ( c != file.end() ) {
-			register char ch = to_lower( *c++ );
+			register char const ch = tolower(
+				iso8859_to_ascii( *c++ )
+			);
 
 			////////// Collect a word /////////////////////////////
 
 			if ( is_word_char( ch ) ) {
 				if ( !in_word ) {
 					// start a new word
-					word_buf[ 0 ] = ch;
-					word_len = 1;
+					word[ 0 ] = ch;
+					len = 1;
 					in_word = true;
 					continue;
 				}
-				if ( word_len < Word_Hard_Max_Size ) {
+				if ( len < Word_Hard_Max_Size ) {
 					// continue same word
-					word_buf[ word_len++ ] = ch;
+					word[ len++ ] = ch;
 					continue;
 				}
 				in_word = false;	// too big: skip chars
-				while ( c != file.end() && is_word_char( *c++ ) ) ;
+				while ( c != file.end() && is_word_char(*c++) );
 				continue;
 			}
 
@@ -423,10 +437,31 @@ stop_word_set*		stop_words;		// pointer to global set
 		////////// Got a word /////////////////////////////////////////
 
 		in_word = false;
-		if ( word_len < Word_Hard_Min_Size )
+		if ( len < Word_Hard_Min_Size )
 			continue;
 
-		word_buf[ word_len ] = '\0';
-		insert( ::strdup( word_buf ) );
+		word[ len ] = '\0';
+		insert( new_strdup( word ) );
 	}
 }
+
+//*****************************************************************************
+//
+// SYNOPSIS
+//
+	stop_word_set::stop_word_set( mmap_file const &index_file )
+//
+// DESCRIPTION
+//
+//	Construct (initialize) a stop_word_set from an existing index file.
+//
+// PARAMETERS
+//
+//	index_file	The index file to read the stop-words from.
+//
+//*****************************************************************************
+{
+	index_segment stop_words( index_file, index_segment::stop_word_index );
+	FOR_EACH( index_segment, stop_words, stop_word )
+		insert( new_strdup( *stop_word ) );
+}
