@@ -8,27 +8,28 @@
 **	it under the terms of the GNU General Public License as published by
 **	the Free Software Foundation; either version 2 of the License, or
 **	(at your option) any later version.
-** 
+**
 **	This program is distributed in the hope that it will be useful,
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **	GNU General Public License for more details.
-** 
+**
 **	You should have received a copy of the GNU General Public License
 **	along with this program; if not, write to the Free Software
 **	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
 // standard
+#include <algorithm>			/* for transform() */
 #include <cctype>
 #include <cstring>
-#include <iostream>
 
 // local
 #include "config.h"
 #include "platform.h"
 #include "token.h"
 #include "util.h"
+#include "word_util.h"
 
 #ifndef	PJL_NO_NAMESPACES
 using namespace std;
@@ -38,31 +39,7 @@ using namespace std;
 //
 // SNYOPSIS
 //
-	/* static */ token* token::hold( token *t )
-//
-// DESCRIPTION
-//
-//	Put back and hold previosuly scanned tokens.  We need somplace to
-//	store tokens that were put back but obviously can't declare a token
-//	static data member inside token directly.
-//
-//*****************************************************************************
-{
-	// Our query parser needs at most 2 look-ahead tokens.
-	static token v[ 2 ];
-	static int top = -1;
-	if ( t ) {
-		v[ ++top ] = *t;
-		return v + top;
-	} else
-		return top >= 0 ? v + top-- : 0;
-}
-
-//*****************************************************************************
-//
-// SNYOPSIS
-//
-	istream& operator>>( istream &in, token &t )
+	token_stream& operator>>( token_stream &tin, token &t )
 //
 // DESCRIPTION
 //
@@ -70,34 +47,35 @@ using namespace std;
 //
 // PARAMETERS
 //
-//	in	The input stream to extract the token from.
+//	tin	The token stream to extract the token from.
 //
 //	t	The newly extracted token.
 //
 // RETURN VALUE
 //
-//	Returns the given input stream as is standard practice.
+//	Returns the first argument as is standard practice.
 //
 // SEE ALSO
 //
 //	Bjarne Stroustrup.  "The C++ Programming Language, 3rd ed."
-//	Addison-Wesley, Reading, MA.  p. 621.
+//	Addison-Wesley, Reading, MA, 1997.  p. 621.
 //
 //*****************************************************************************
 {
-	token *const hold = token::hold();
-	if ( hold ) {
+	token *const put_back = tin.put_back();
+	if ( put_back ) {
 		//
 		// If there was a token previously put back, use it.
 		//
-		t = *hold;
-		return in;
+		t = *put_back;
+		return tin;
 	}
 	t.type_ = token::no_token;
 	bool in_word = false;
 	char c;
 
-	while ( in.get( c ) ) {
+	while ( tin.get( c ) ) {
+		c = iso8859_to_ascii( c );
 
 		if ( is_word_char( c ) ) {
 			if ( !in_word ) {
@@ -113,7 +91,7 @@ using namespace std;
 				continue;
 			}
 			in_word = false;		// too big: skip chars
-			while ( in.get( c ) && is_word_char( c ) ) ;
+			while ( tin.get( c ) && is_word_char( c ) ) ;
 			continue;
 		}
 
@@ -121,28 +99,31 @@ using namespace std;
 			if ( c == '*' )
 				t.type_ = token::word_star_token;
 			else
-				in.putback( c );
+				tin.putback( c );
 			break;
 		}
 
 		switch ( c ) {
 			case '(':
 				t.type_ = token::lparen_token;
-				return in;
+				return tin;
 			case ')':
 				t.type_ = token::rparen_token;
-				return in;
+				return tin;
 			case '=':
 				t.type_ = token::equal_token;
-				return in;
+				return tin;
 		}
 	}
 
 	if ( in_word ) {
 		t.buf_[ t.len_ ] = '\0';
-		::strcpy( t.lower_buf_, to_lower( t.buf_ ) );
+		::transform(
+			t.buf_, t.buf_ + t.len_ + 1,
+			t.lower_buf_, static_cast<char (*)(char)>( to_lower )
+		);
 		if ( t.type_ )
-			return in;
+			return tin;
 
 		//
 		// Check to see if the word is an operator.  For only 3
@@ -158,5 +139,5 @@ using namespace std;
 			t.type_ = token::word_token;
 	}
 
-	return in;
+	return tin;
 }
