@@ -8,12 +8,12 @@
 **	it under the terms of the GNU General Public License as published by
 **	the Free Software Foundation; either version 2 of the License, or
 **	(at your option) any later version.
-** 
+**
 **	This program is distributed in the hope that it will be useful,
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **	GNU General Public License for more details.
-** 
+**
 **	You should have received a copy of the GNU General Public License
 **	along with this program; if not, write to the Free Software
 **	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -24,28 +24,49 @@
 
 // standard
 #include <cctype>
+#include <cerrno>
+#include <climits>
 #include <cstring>
+#include <iostream>
 #include <string>
+#include <ctime>			/* needed by sys/resource.h */
+#include <sys/time.h>			/* needed by FreeBSD systems */
+#include <sys/resource.h>
 #include <sys/stat.h>
+#include <unistd.h>			/* for _exit(2), geteuid(2) */
+
+//
+// POSIX.1 is, IMHO, brain-damaged in the way it makes you determine the
+// maximum path-name length, so we'll simply pick a sufficiently large constant
+// such as 1024.  In practice, this is the actual value used on many SVR4 as
+// well as 4.3+BSD systems.
+//
+// See also: W. Richard Stevens.  "Advanced Programming in the Unix
+// Environment," Addison-Wesley, Reading, MA, 1993.  pp. 34-42.
+//
+#ifdef	PATH_MAX
+#undef	PATH_MAX
+#endif
+int const		PATH_MAX = 1024;
 
 // local
-#include "config.h"
-#include "file_vector.h"
-#include "platform.h"				/* for PJL_NO_SYMBOLIC_LINKS */
+#include "exit_codes.h"
+#include "fake_ansi.h"			/* for std */
+#include "platform.h"			/* for PJL_NO_SYMBOLIC_LINKS */
 
-extern struct stat	stat_buf;		// somplace to do a stat(2) in
+extern char const*	me;
 
 //*****************************************************************************
 //
 // SYNOPSIS
 //
-	template< int Size, int N > class char_buffer_pool
+	template< int Buf_Size, int N > class char_buffer_pool
 //
 // DESCRIPTION
 //
 //	A char_buffer_pool maintains a small set ("pool") of size N of
-//	available character buffers, each of size Size, and issues them in a
-//	round-robin manner.
+//	available character buffers, each of size Buf_Size, and issues them in
+//	a round-robin manner.
 //
 //	This is used by functions to return a character string without having
 //	to allocate memory dynamically nor have previously returned strings
@@ -63,7 +84,7 @@ public:
 			return cur_buf_;
 		}
 private:
-	char	buf_[ N ][ Size ];
+	char	buf_[ N ][ Buf_Size ];
 	int	next_buf_index_;
 	char	*cur_buf_;
 };
@@ -72,158 +93,131 @@ private:
 //
 // SYNOPSIS
 //
-	inline bool is_html_ext( char const *ext )
+	inline char const* find_newline( char const *c, char const *end )
 //
 // DESCRIPTION
 //
-//	Checks the given filename extension to see if it is one used for HTML
-//	files: html, htm, or shtml.  Case is significant.
+//	Finds the next newline (CR or LF) in a character sequence.
 //
 // PARAMETERS
 //
-//	ext	The filename extension to be checked.
+//	c	The iterator to use.
+//
+//	end	The iterator marking the end of the range to check.
 //
 // RETURN VALUE
 //
-//	Returns true only if the filename extension is one used for HTML
-//	files.
+//	Returns an iterator positioned either at the first character of a CR-LF
+//	if found, or at "end" if not.
 //
 //*****************************************************************************
 {
-	int const i = ext[0] == 's';		// This is faster than calling
-	return	 ext[ i ] == 'h' &&		// strcmp() multiple times.
-		 ext[i+1] == 't' &&
-		 ext[i+2] == 'm' &&
-		(ext[i+3] == 'l' && !ext[i+4] ||
-		!ext[i+3] && !i);
+	while ( c != end && *c != '\n' && *c != '\r' )
+		++c;
+	return c;
 }
 
 //*****************************************************************************
 //
 // SYNOPSIS
 //
-	inline bool is_vowel( char c )
+	template< class T > void max_out_limit( T resource )
 //
 // DESCRIPTION
 //
-//	Determine whether a character is a lower-case vowel [aeiou].
+//	Set the limit for the given resource to its maximum value.  If we're
+//	running as root, set it to infinity.
 //
 // PARAMETERS
 //
-//	c	The character to be checked.
+//	resource	The ID for the resource as given in sys/resources.h.
 //
-// RETURN VALUE
+// NOTE
 //
-//	Returns true only if the character is a vowel.
+//	This can't be an ordinary function since the type "resource" isn't int
+//	on some systems (e.g., Linux).
+//
+// SEE ALSO
+//
+//	W. Richard Stevens.  "Advanced Programming in the Unix Environment,"
+//	Addison-Wesley, Reading, MA, 1993. pp. 180-184.
 //
 //*****************************************************************************
 {
-	return c == 'a' || c == 'e' || c == 'i' || c == 'o' || c == 'u';
+	struct rlimit r;
+#ifdef	RLIM_INFINITY	/* hey, you never know: it might be undefined */
+	if ( ::geteuid() == 0 )
+		r.rlim_max = RLIM_INFINITY;
+	else
+#endif
+		::getrlimit( resource, &r );
+	r.rlim_cur = r.rlim_max;
+	::setrlimit( resource, &r );
 }
 
 //*****************************************************************************
 //
 // SYNOPSIS
 //
-	inline bool is_word_char( char c )
+	inline char const* pjl_basename( char const *file_name )
 //
 // DESCRIPTION
 //
-//	Check whether a given character is a "word character," one that is
-//	valid to be in a word.
+//	Determine the base name of a given file name.
 //
 // PARAMETERS
 //
-//	c	The character to be checked.
+//	file_name	The file_name.
 //
 // RETURN VALUE
 //
-//	Returns true only if the character is a "word character."
+//	Returns a pointer to the base name of the file name.  Note that the
+//	pointer points within file_name, i.e., the two will share storage.
 //
 //*****************************************************************************
 {
-	return c > 0 &&
-#	if OPTIMIZE_WORD_CHARS
-	( isalnum( c ) ||
-		//
-		// If you change Word_Chars in config.h from the default set
-		// but would like to keep the optimization, edit the line below
-		// to compare 'c' against every non-alphanumeric character in
-		// your set of Word_Chars.
-		//
-		c == '&' || c == '\'' || c == '-' || c == '_'
-	);
-#	else
-	std::strchr( Word_Chars, tolower( c ) ) != 0;
-#	endif
+	char const *const slash = ::strrchr( file_name, '/' );
+	return slash ? slash + 1 : file_name;
 }
 
 //*****************************************************************************
 //
 // SYNOPSIS
 //
-	inline bool is_word_begin_char( char c )
+	inline char const* skip_newline( char const *c, char const *end )
 //
 // DESCRIPTION
 //
-//	Check whether a given character is a "word beginning character," one
-//	that is valid to begin a word.
+//	Skips the next newline (CR and/or LF) in a character sequence.
 //
 // PARAMETERS
 //
-//	c	The character to be checked.
+//	c	The iterator positioned at either a CR or LF.
+//
+//	end	The iterator marking the end of the range.
 //
 // RETURN VALUE
 //
-//	Returns true only if the character is a "word beginning character."
+//	Returns an iterator positioned either at a character after a newline if
+//	found, or untouched if not.
 //
 //*****************************************************************************
 {
-#	if OPTIMIZE_WORD_BEGIN_CHARS
-	//
-	// If you change Word_Begin_Chars in config.h from the default set but
-	// would like to keep the optimization, edit the line below to compare
-	// 'c' against every character in your set of Word_Begin_Chars.
-	//
-	return isalnum( c );
-#	else
-	return std::strchr( Word_Begin_Chars, tolower( c ) ) != 0;
-#	endif
+	if ( c != end && *c == '\r' )
+		++c;
+	if ( c != end && *c == '\n' )
+		++c;
+	return c;
 }
 
 //*****************************************************************************
 //
-// SYNOPSIS
-//
-	inline bool is_word_end_char( char c )
-//
-// DESCRIPTION
-//
-//	Check whether a given character is a "word ending character," one
-//	that is valid to end a word.
-//
-// RETURN VALUE
-//
-//	Returns true only if the character is a "word ending character."
+//	File test functions.  Those that do not take an argument operate on the
+//	last file stat'ed.
 //
 //*****************************************************************************
-{
-#	if OPTIMIZE_WORD_END_CHARS
-	//
-	// Same deal as with OPTIMIZE_WORD_BEGIN_CHARS.
-	//
-	return isalnum( c );
-#	else
-	return std::strchr( Word_End_Chars, tolower( c ) ) != 0;
-#	endif
-}
 
-//*****************************************************************************
-//
-//	File test functions.  Those that do not take an argument operate on
-//	the last file stat'ed.
-//
-//*****************************************************************************
+extern struct stat	stat_buf;		// somplace to do a stat(2) in
 
 inline bool	file_exists( char const *path ) {
 			return ::stat( path, &stat_buf ) != -1;
@@ -232,6 +226,8 @@ inline bool	file_exists( std::string const &path ) {
 			return file_exists( path.c_str() );
 		}
 
+inline off_t	file_size() { return stat_buf.st_size; }
+
 inline bool	is_directory() {
 			return S_ISDIR( stat_buf.st_mode );
 		}
@@ -243,7 +239,7 @@ inline bool	is_directory( std::string const &path ) {
 		}
 
 inline bool	is_plain_file() {
-			return S_ISREG( stat_buf.st_mode & S_IFMT );
+			return S_ISREG( stat_buf.st_mode );
 		}
 inline bool	is_plain_file( char const *path ) {
 			return file_exists( path ) && is_plain_file();
@@ -271,28 +267,43 @@ inline bool	is_symbolic_link( std::string const &path ) {
 //
 //*****************************************************************************
 
-#define	ERROR		cerr << me << ": error: "
-
-extern void		get_index_info(
-				file_vector<char> const &file, int i,
-				long *n, off_t const **offset
-			);
-
-extern bool		is_ok_word( char const *word );
-
-inline char*		new_strdup( char const *s ) {
-				return ::strcpy( new char[ ::strlen(s)+1 ], s );
+inline std::ostream&	error( std::ostream &o = std::cerr ) {
+				return o << me << ": error: ";
+			}
+inline std::ostream&	error_string( std::ostream &o = std::cerr ) {
+				return o << ": " << std::strerror( errno )
+					<< std::endl;
 			}
 
-extern void		parse_config_file( char const *file_name );
+#define	internal_error	std::cerr << (me ? me : "SWISH++") << ", \"" \
+			<< __FILE__ << "\", line " << __LINE__ \
+			<< ": internal error:\n"
+
+inline std::ostream&	report_error( std::ostream &o = std::cerr ) {
+				o << "please report this error\n";
+				::_exit( Exit_Internal_Error );
+				return o;	// just to make compiler happy
+			}
+
+inline char*		new_strdup( char const *s ) {
+				return std::strcpy(
+					new char[ std::strlen( s ) + 1 ], s
+				);
+			}
 
 			// ensure function semantics: 'c' is expanded once
+inline bool		is_alnum( char c )	{ return isalnum( c ); }
+inline bool		is_alpha( char c )	{ return isalpha( c ); }
+inline bool		is_digit( char c )	{ return isdigit( c ); }
+inline bool		is_punct( char c )	{ return ispunct( c ); }
+inline bool		is_space( char c )	{ return isspace( c ); }
+inline bool		is_upper( char c )	{ return isupper( c ); }
+inline bool		is_xdigit( char c )	{ return isxdigit( c ); }
 inline char		to_lower( char c )	{ return tolower( c ); }
-extern char*		to_lower( char const *s );
+extern char*		to_lower( char const* );
+extern char*		to_lower_r( char const* );
 extern char*		to_lower( char const *begin, char const *end );
-
-extern char const*	ltoa( long );
-inline char const*	itoa( int n )		{ return ::ltoa( n ); }
+extern char*		to_lower_r( char const *begin, char const *end );
 
 #define	FOR_EACH(T,C,I) \
 	for ( T::const_iterator I = (C).begin(); I != (C).end(); ++I )
