@@ -8,12 +8,12 @@
 **	it under the terms of the GNU General Public License as published by
 **	the Free Software Foundation; either version 2 of the License, or
 **	(at your option) any later version.
-** 
+**
 **	This program is distributed in the hope that it will be useful,
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **	GNU General Public License for more details.
-** 
+**
 **	You should have received a copy of the GNU General Public License
 **	along with this program; if not, write to the Free Software
 **	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -29,8 +29,8 @@
 #include <string>
 
 // local
-#include "fake_ansi.h"				/* for explicit, std */
-#include "html.h"				/* for No_Meta_ID */
+#include "fake_ansi.h"			/* for explicit, std */
+#include "indexer.h"			/* for No_Meta_ID */
 #include "my_set.h"
 
 //*****************************************************************************
@@ -48,26 +48,32 @@
 {
 public:
 	struct file {
-		typedef my_set< short > meta_set;
+		typedef PJL::my_set< short > meta_set;
 
 		int		index_;		// occurs in i-th file
 		meta_set	meta_ids_;	// meta name(s) associated with
-		union {
-			short	occurrences_;	// in this file only
-			short	rank_;
-		};
+		short		occurrences_;	// in this file only
+		short		rank_;
 
 		file() { }
-		explicit file( int i ) : index_( i ), occurrences_( 1 ) { }
-		file( int i, int mi ) : index_( i ), occurrences_( 1 ) {
-			if ( mi != No_Meta_ID )
-				meta_ids_.insert( mi );
+		explicit file( int index ) :
+			index_( index ), occurrences_( 1 ), rank_( 0 ) { }
+		file( int index, int meta_id ) :
+			index_( index ), occurrences_( 1 ), rank_( 0 )
+		{
+			if ( meta_id != No_Meta_ID )
+				meta_ids_.insert( meta_id );
 		}
+
+		// default "file( file const& )" is ok
+		// default "file& operator=( file const& )" is ok
+
+		void	write_meta_ids( std::ostream& ) const;
 	};
 
-	typedef std::list< file > file_set;
+	typedef std::list< file > file_list;
 
-	file_set	files_;
+	file_list	files_;			// the files this word is in
 	int		occurrences_;		// over all files
 
 	word_info() : occurrences_( 0 ) { }
